import logging
import os
from datetime import datetime
from poaupdater import uLogging, uTextRender

LOG_DIR = "/var/log/pa/vps-migration/"
SUBSCRIPTION_LOG = "migration-{}.log"
COMMON_LOG = "migration.log"
SUMMARY_LOG = "summary.log"


def setup_logging(file, level):
    if not os.path.exists(LOG_DIR):
        os.makedirs(LOG_DIR)
        
    log_format = '%(asctime)-15s [%(levelname)s] [%(process)d:%(processName)s] %(message)s'
    default_logger = logging.getLogger()
    default_logger.setLevel(level)

    debug_handler = logging.FileHandler(file, "w+")
    debug_handler.setLevel(logging.DEBUG)
    debug_handler.setFormatter(logging.Formatter(log_format))

    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(logging.Formatter(log_format))

    default_logger.addHandler(debug_handler)
    default_logger.addHandler(console_handler)
    default_logger.log_file_stream = None

    uLogging.init_external(default_logger)


def create_result_log(migrate_subscription_summary):
    path = os.path.join(LOG_DIR, SUMMARY_LOG)
    uLogging.init2(path, True, True)
    table = uTextRender.Table()
    table.setHeader(["Subscription Id", "Target Plan Id", "Target Plan Period Id", "Target Service Template Id",
                     "Target Subscription Id", "Status"])
    summary_values = migrate_subscription_summary.values()
    summary_values.sort(key=lambda value: value["status"])
    
    for subscription in summary_values:
        table.addRow([subscription.get("subscrId"),
                      subscription.get("targetPlanId"),
                      subscription.get("targetPlanPeriodId"),
                      subscription.get("targetServiceTemplateId"),
                      subscription.get("newSubscrId"),
                      subscription.get("status")])

    uLogging.info("\n%s\n" % table)
    uLogging.info("Result log is saved in " + path)


def write_summary(summary):
    failed = len(list(filter(lambda summary: summary['status'] == 'Failed', summary.values())))
    successful = len(list(filter(lambda summary: summary['status'] == 'Ok', summary.values())))
    not_processed = len(list(filter(lambda summary: summary['status'] == 'Not Processed', summary.values())))
    total = len(summary.values())
    uLogging.info("Total: {}, Successful: {}, Failed: {}, Not Processed: {}".format(total, successful, failed, not_processed))


def subscription_log_path(subscription_id):
    return os.path.join(LOG_DIR, SUBSCRIPTION_LOG.format(subscription_id))


def common_log_path():
    return os.path.join(LOG_DIR, COMMON_LOG)


class Logger:
    def __init__(self, subscription_id):
        self.subscription_id = subscription_id

    def debug(self, msg):
        uLogging.debug("[SubId: {}] {}".format(self.subscription_id, msg))

    def info(self, msg):
        uLogging.info("[SubId: {}] {}".format(self.subscription_id, msg))

    def warn(self, msg):
        uLogging.warn("[SubId: {}] {}".format(self.subscription_id, msg))
